import bpy

def add_modifier_armature(rig, obj) -> None:
    obj.modifiers.new(name = "Skeleton", type = "ARMATURE")
    obj.modifiers["Skeleton"].object = rig

def add_modifier_lattice_smart_deform(rig, obj) -> None:
    lattice = rig.children[0].modifiers["Lattice_Smart_Deform"].object
    obj.modifiers.new(name = "deform", type = "LATTICE")
    obj.modifiers["deform"].object = lattice

def add_modifier_lattice_head(rig, obj) -> None:
    lattice = rig.children[0].modifiers["Lattice_Head"].object
    obj.modifiers.new(name = "squash stretch", type = "LATTICE")   
    obj.modifiers["squash stretch"].object = lattice

def add_modifier_solidify(obj, thickness) -> None:
    obj.modifiers.new(name = "Solidify", type = "SOLIDIFY")
    obj.modifiers["Solidify"].thickness = thickness
    obj.modifiers["Solidify"].use_even_offset = True
    obj.modifiers["Solidify"].use_quality_normals = True

def add_modifier_subdivision(obj, viewport_level, render_level) -> None:
    obj.modifiers.new(name = "subdivision", type = "SUBSURF")
    obj.modifiers['subdivision'].levels = viewport_level
    obj.modifiers['subdivision'].render_levels = render_level
    obj.modifiers['subdivision'].subdivision_type = "SIMPLE"

def add_modifier_lattice_chest(rig, obj) -> None:
    lattice = rig.children[0].modifiers["Chest"].object
    obj.modifiers.new(name = "Chest", type = "LATTICE")
    obj.modifiers["Chest"].object = lattice
    obj.modifiers["Chest"].vertex_group = "UpperBodyMain"

def create_item(key):
    return (key, key, "")

def get_rig():
    scn = bpy.context.scene
    if scn.get('thomas_rig_legacy'):
        if scn.thomas_rig_legacy.get('reference'):
            return scn.thomas_rig_legacy.reference
    return bpy.context.active_object

def is_packed(img):
    try:
        return img.packed_files.values() != []
    except (AttributeError, KeyError, TypeError):
        return False

def safe_unpack(img): #unpacks relative to the file ONLY if the file is saved
    if img.packed_files:
        if bpy.data.is_saved:
            return img.unpack()
        else:
            return img.unpack(method='USE_ORIGINAL')
        

class UI_Utils:
    def spacer(layout, factor):
        spacer = layout.row()
        spacer.label(text="")
        spacer.scale_y = factor

    def check_search(display, prop_src, arm_filter=False) -> bool:
        search = get_rig().pose.bones["Misc_Properties"]["search"].lower()
        if arm_filter:
            arm_props = {"L.Arm_FK/IK", "R.Arm_FK/IK", "L.Arm_World", "R.Arm_World"}

            for prop in prop_src.keys():
                if prop in arm_props and search in prop.lower():
                    display.prop(prop_src, f'["{prop}"]', toggle = True)
            return

        for prop in prop_src.keys():
            if(search in prop.lower() and search != ""):
                display.prop(prop_src, f'["{prop}"]', toggle = True)